@@ -731,7 +731,7 @@ def compute_similarity(data, weights, p=2):
     df2 = data.groupby('time')
     df3 = pd.DataFrame()  # empty df in which all the data frames containing the distance are merged
     for start in tqdm(df2.groups.keys(), position=0, desc="Computing euclidean distance"):
-        groups_df = df2.get_group(start).groupby('time').apply(similarity_computation, w=w,
+        groups_df = df2.get_group(start).groupby('time', group_keys=False).apply(similarity_computation, w=w,
                                                                p=p)  # calculate distance for each time period
         df3 = pd.concat([df3, groups_df])  # finally all dataframes are merged in df3
 
@@ -760,7 +760,7 @@ def compute_similarity_multiproccessing(data, weights, p=2):
     df2 = data.groupby('time')
     df3 = pd.DataFrame()  # empty df in which all the data frames containing the distance are merged
     for start in tqdm(df2.groups.keys(), position=0, desc="Computing euclidean distance", disable=True):
-        groups_df = df2.get_group(start).groupby('time').apply(similarity_computation, w=w,
+        groups_df = df2.get_group(start).groupby('time', group_keys=False).apply(similarity_computation, w=w,
                                                                p=p)  # calculate distance for each time period
         df3 = pd.concat([df3, groups_df])  # finally all dataframes are merged in df3
 
@@ -846,12 +846,12 @@ def ts_feature(data, feature):
 def explore_features(data):
     """
     Show percentage of environment space explored by singular animal.
-    Using minumum and maximum of 2-D coordinates, given by 'x' and 'y' features in input DataFrame.
+    Using minumum and maximum of coordinates, given by 'x' and 'y' (and 'z') features in input DataFrame.
     :param data: pandas DataFrame, containing preprocessed movement records.
     :return: None.
     """
     # Compute global minimum and maximum if user
-    # has NOT specified the values-
+    # has NOT specified the values-^
     x_min = data['x'].min()
     x_max = data['x'].max()
 
@@ -862,6 +862,8 @@ def explore_features(data):
         is_3d = True
         z_min = data['z'].min()
         z_max = data['z'].max()
+    else:
+        is_3d = False
 
     # Group 'data' using 'animal_id' attribute-
     data_groups = grouping_data(data)
@@ -876,17 +878,13 @@ def explore_features(data):
         aid_y_max = data_groups[aid]['y'].max()
 
         print("\nAnimal ID: {0} covered % of area:".format(aid))
-        print("x-coordinates: minimum = {0:.2f}% & maximum = {1:.2f}%".format(
-            (x_min / aid_x_min) * 100, (aid_x_max / x_max) * 100))
-
-        print("y-coordinates: minimum = {0:.2f}% & maximum = {1:.2f}%".format(
-            (y_min / aid_y_min) * 100, (aid_y_max / y_max) * 100))
+        print(f'x-coordinates: {round(((aid_x_max-aid_x_min)/(x_max - x_min))*100, 2)}')
+        print(f'y-coordinates: {round(((aid_y_max - aid_y_min) / (y_max - y_min))*100, 2)}')
 
         if is_3d:
             aid_z_min = data_groups[aid]['z'].min()
             aid_z_max = data_groups[aid]['z'].max()
-            print("z-coordinates: minimum = {0:.2f}% & maximum = {1:.2f}%".format(
-                (z_min / aid_z_min) * 100, (aid_z_max / z_max) * 100))
+            print(f'z-coordinates: {round(((aid_z_max - aid_z_min) / (z_max - z_min))*100, 2)}')
 
     return None
 
@@ -1209,7 +1207,7 @@ def hausdorff_distance(data, mover1=None, mover2=None):
                     hdf_distance = max(hdf_distance1, hdf_distance2)
                 dict[aid2] = hdf_distance
             aid_df = pd.DataFrame(dict, index=[aid1])
-            df = df.append(aid_df)
+            df = pd.concat([df, aid_df])
 
         return df
 
